/*
 * Esta classe representa as notas existentes no ArquivoMidi
 */

package classificador;

/**
 *
 * @author dj
 */
class Nota {

    public static final String[] sNotas = {"C",  "C#", "D", "Eb", "E", "F",
                                           "F#", "G" ,"G#", "A", "Bb", "B"  };
    private int numeroDaNota;
    private String nota;
    private int oitava;
    private long tickInicial;
    private long tickFinal;
    private long ticks;

    /**
     * Construtor baseado no numero da nota
     * @param nNota
     */
    public Nota(int nNota) {
        numeroDaNota = nNota;
        if(numeroDaNota >=0 && numeroDaNota <= 127){
            nota = sNotas[numeroDaNota % 12];
            oitava = (numeroDaNota / 12) - 1;
        }
        else{
            nota = null;
            oitava = 0;
            tickInicial = tickFinal = ticks = 0;
        }
    }

    /**
     * Construtor baseado no numero da nota e seu ponto de inicio
     * @param nNota
     * @param thinkIn
     */
    public Nota(int nNota, long thinkIn) {
        numeroDaNota = nNota;
        if(numeroDaNota >=0 && numeroDaNota <= 127){
            nota = sNotas[numeroDaNota % 12];
            oitava = (numeroDaNota / 12) - 1;
            tickInicial = thinkIn;
        }
        else{
            nota = null;
            oitava = 0;
            tickInicial = tickFinal = ticks = 0;
        }
    }

    /**
     * Checa se duas notas sao iguais
     */
    public boolean isEquals(Object o){
        if(this.getNumeroDaNota() == ((Nota) o).getNumeroDaNota())
        {
                return true;
        }
        else    return false;

    }

    /**
     * Imprime a nota no formato:
     *
     *          Nota+Oitava
     */
    public void imprimirNota(){
        System.out.print("Nota: "+nota+oitava);
    }

    /**
     * Imprime as informacoes da nota no formato:
     *
     *          Nota+Oitava, tickInicial, tickFinal, duracao
     */
    public void imprimirNotaInfo(){
        System.out.println("Nota: "+nota+oitava+
                           ", tickInicial: "+tickInicial+
                           ", tickFinal: "+tickFinal+
                           ", duracao: "+ticks+" ticks");
    }

    /**
     * @return the numeroDaNota
     */
    public int getNumeroDaNota() {
        return numeroDaNota;
    }

    /**
     * @return the nota
     */
    public String getNota() {
        return nota;
    }

    /**
     * @return the oitava
     */
    public int getOitava() {
        return oitava;
    }

    /**
     * @param tickInicial the tickInicial to set
     */
    public void setTickInicial(long tickInicial) {
        this.tickInicial = tickInicial;
    }

    /**
     * @param tickFinal the tickFinal to set
     */
    public void setTickFinal(long tickFinal) {
        this.tickFinal = tickFinal;
        this.ticks = this.getTickFinal() - this.getTickInicial();
    }

    /**
     * @return the tickInicial
     */
    public long getTickInicial() {
        return tickInicial;
    }

    /**
     * @return the tickFinal
     */
    public long getTickFinal() {
        return tickFinal;
    }

    /**
     * @return the ticks
     */
    public long getTicks() {
        return ticks;
    }
    
}
