/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package classificador;

import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;



// Copyright Wikipedia com alteracoes para ficar invertido
/**
 * Esta classe representa os resultados da validacao cruzada.
 * As informacoes armazenadas aqui servem para representacao dos resultados e
 * realizacao de novos calculos a partir deles.
 *
 * @author dj
 */
public class Resultado {

    /**
     * Lista contendo os acertos de cada modelo
     */
    private String titulo;
    private int contexto;
    private boolean modeloMelodico;
    private boolean modeloRitmico;
    private int quantidadeDeCompositores;
    private int quantidadeDeArquivosPorCompositor;
    private int[] acertosPorGrupo;
    private int[] acertosPorCompositor;
    private ItemResultado[][] resultados;
    private int contadorDeResultados = 0;
    private String []nomeCompositores;

    public Resultado(){

    }
    
    
    public Resultado(String titulo, int contexto, boolean modMelodico, boolean modRitmico, int qntCompositores, int qntArquivosPorCompositor){
        this.titulo = titulo;
        this.contexto = contexto;
        this.modeloMelodico = modMelodico;
        this.modeloRitmico = modRitmico;
        this.quantidadeDeCompositores = qntCompositores;
        this.nomeCompositores = new String[quantidadeDeCompositores];
        this.quantidadeDeArquivosPorCompositor = qntArquivosPorCompositor;

        this.acertosPorCompositor = new int[qntCompositores];

        this.resultados = new ItemResultado[qntCompositores*qntArquivosPorCompositor][qntCompositores+1];
    }

    public Resultado(String titulo, int contexto, boolean modMelodico, boolean modRitmico, int qntCompositores, int qntArquivosPorCompositor, int qntItensPorResultado){
        this.titulo = titulo;
        this.contexto = contexto;
        this.modeloMelodico = modMelodico;
        this.modeloRitmico = modRitmico;
        this.quantidadeDeCompositores = qntCompositores;
        this.nomeCompositores = new String[quantidadeDeCompositores];
        this.quantidadeDeArquivosPorCompositor = qntArquivosPorCompositor;

        this.acertosPorCompositor = new int[qntCompositores];

        this.resultados = new ItemResultado[qntCompositores*qntArquivosPorCompositor][qntItensPorResultado];
    }
    
    public void inserirResultado(LinkedList<ItemResultado> resultadoArquivoAtual) {

        for(int i = 0; i < resultadoArquivoAtual.size(); i++){
            resultados[contadorDeResultados][i] = resultadoArquivoAtual.get(i);
        }
        contadorDeResultados++;
    }

    public void ordenarResultados(){
        
        for(int i = 0; i < contadorDeResultados; i++){
            
            QuickSortResultado.quick_sort(getResultados()[i], 1, quantidadeDeCompositores+1);
            
            //faz uma ultima checagem para verificar se o autor ficou
            //com a mesma taxa do primeiro lugar
            for(int j = 1; j <= quantidadeDeCompositores; j++){
                if(getResultados()[i][0].getCompositorItem() == getResultados()[i][j].getCompositorItem()){
                    if( j != 1 ){
                        if(getResultados()[i][j].getValor() == getResultados()[i][1].getValor()){
                            ItemResultado temp = getResultados()[i][1].clone();
                            getResultados()[i][1] = getResultados()[i][j].clone();
                            getResultados()[i][j] = temp.clone();
                        }
                    }
                    break;
                }
            }
        }
    }
    
    // metodo util para a classificacao v4
    public void ordenarCompositoresResultados(){
        
        for(int i = 0; i < contadorDeResultados; i++){
            QuickSortCompositor.quick_sort(getResultados()[i], 1, quantidadeDeCompositores+1);
        }
    }
    
    // metodo util para a classificacao v4
    public void trocarTaxaPorPosicaoNoRanking(){
        
        double razaoTemp = Double.MIN_VALUE;
        int posicaoTemp = Integer.MIN_VALUE;
        for(int i = 0; i < resultados.length; i ++){
            for(int j = 1; j < resultados[i].length; j++){
                if(resultados[i][j].getValor() != razaoTemp){
                    razaoTemp = resultados[i][j].getValor();
                    resultados[i][j].setValor(j);
                    posicaoTemp = j;
                }else{
                    resultados[i][j].setValor(posicaoTemp);
                }
            }
            razaoTemp = Double.MIN_VALUE;
            posicaoTemp = Integer.MIN_VALUE;
        }
    }
    
    public void calcularAcertos(){
        
        System.out.println("\nCalculando acertos:");

        for(int i = 0; i < contadorDeResultados; i++){
            if(getResultados()[i][0].getCompositorItem() == getResultados()[i][1].getCompositorItem()){
                this.acertosPorCompositor[getResultados()[i][0].getCompositorItem()]++;
                System.out.print(Integer.toString(getResultados()[i][0].getCompositorItem()));
            }
        }
        System.out.println("");
    }

    public void exportarDescricao(FileWriter saida){
        try {
            saida.write("\nResultado;\n");
            saida.write("Teste:PPM+" + this.titulo +";\n");
            saida.write("Contexto:" + this.contexto +";\n");
            saida.write("Melodico:" + this.modeloMelodico +";\n");
            saida.write("Ritmico:" + this.modeloRitmico +";\n");
            saida.write("\n");
        } catch (IOException ex) {
            System.out.println("Erro ao tentar escrever no arquivo de saida");
            Logger.getLogger(Resultado.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void exportarAcertos(FileWriter saida){
        try {
            saida.write("Acertos por compositor;\n");
            // i = compositor atual
            for(int i = 0; i < quantidadeDeCompositores; i++){
                saida.write(i+"-"+getNomeCompositores()[i]+";"+acertosPorCompositor[i]+";\n");
            }
            saida.write("\n");
            
        } catch (IOException ex) {
            System.err.println("Erro ao tentar exportar resultado");
            Logger.getLogger(Resultado.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void exportarListaResultados(FileWriter saida){
        try {
            saida.write("Lista de resultados:\n");
            for(int i = 0; i < contadorDeResultados; i++){
                String tempRes = "";
                for(ItemResultado item : getResultados()[i]){
                    tempRes = tempRes.concat(Integer.toString(item.getCompositorItem()));
                    tempRes = tempRes.concat(":");
                    tempRes = tempRes.concat(item.getNomeDoArquivo());
                    tempRes = tempRes.concat(":");
                    tempRes = tempRes.concat(Double.toString(item.getValor()));
                    tempRes = tempRes.concat(";");
                }
                saida.write(tempRes+"\n");
            }
            
        } catch (IOException ex) {
            System.err.println("Erro ao tentar exportar resultado.");
            Logger.getLogger(Resultado.class.getName()).log(Level.SEVERE, null, ex);
        }

    }

    /**
     * @return the contexto
     */
    public int getContexto() {
        return contexto;
    }

    /**
     * @return the modeloMelodico
     */
    public boolean isModeloMelodico() {
        return modeloMelodico;
    }

    /**
     * @return the modeloRitmico
     */
    public boolean isModeloRitmico() {
        return modeloRitmico;
    }

    /**
     * @return the quantidadeDeModelos
     */
    public int getQuantidadeDeCompositores() {
        return quantidadeDeCompositores;
    }

    /**
     * @return the quantidadeDeArquivosPorCompositor
     */
    public int getQuantidadeDeArquivosPorCompositor() {
        return quantidadeDeArquivosPorCompositor;
    }

    /**
     * @return the acertosPorModelo
     */
    public int[] getAcertosPorCompositor() {
        return acertosPorCompositor;
    }

    /**
     * @return the resultados
     */
    public ItemResultado[][] getResultados() {
        return resultados;
    }

    /**
     * @return the nomeCompositores
     */
    public String[] getNomeCompositores() {
        return nomeCompositores;
    }

    /**
     * @param nomeCompositores the nomeCompositores to set
     */
    public void setNomeCompositores(String[] nomeCompositores) {
        this.nomeCompositores = nomeCompositores;
    }

}

//Copyright Wikipedia + algumas alteracoes
class QuickSortResultado{
    
    public static void quick_sort(ItemResultado []v, int ini, int fim){
            int meio;

            if(ini<fim){
                    meio = particao(v,ini,fim);
                    quick_sort(v,ini,meio);
                    quick_sort(v,meio+1,fim);
            }
    }
    
    public static int particao(ItemResultado []v, int ini, int fim){
            ItemResultado pivo;
            int topo;
            int i;
            pivo = v[ini].clone();
            topo = ini;

            for(i=ini+1;i<fim;i++){
                    if( v[i].getValor() < pivo.getValor()){
                            v[topo] = v[i].clone();
                            v[i] = v[topo+1].clone();
                            topo++;        
                    }
            }
            v[topo] = pivo.clone();
            return topo;
    }
    
}


//Copyright Wikipedia + algumas alteracoes
class QuickSortCompositor{
    
    public static void quick_sort(ItemResultado []v, int ini, int fim){
            int meio;

            if(ini<fim){
                    meio = particao(v,ini,fim);
                    quick_sort(v,ini,meio);
                    quick_sort(v,meio+1,fim);
            }
    }
    
    public static int particao(ItemResultado []v, int ini, int fim){
            ItemResultado pivo;
            int topo;
            int i;
            pivo = v[ini].clone();
            topo = ini;

            for(i=ini+1;i<fim;i++){
                    if( v[i].getCompositorItem() < pivo.getCompositorItem()){
                            v[topo] = v[i].clone();
                            v[i] = v[topo+1].clone();
                            topo++;        
                    }
            }
            v[topo] = pivo.clone();
            return topo;
    }
    
}
